\name{getJPEGDimensions}
\alias{getJPEGDimensions}
\title{Compute the dimensions of JPEG images}
\description{
  This function reads the header information from
  the specified files and returns the dimensions of the
  JPEG images
}
\usage{
getJPEGDimensions(filenames, failOnError = FALSE)
}
\arguments{
  \item{filenames}{a character vector identifying the JPEG files.}
  \item{failOnError}{a logical value indicating whether to raise an
    error (\code{TRUE}) or simply a warning (\code{FALSE})
    if a file is not readable or "corrupted", i.e. not a valid
    JPEG file.}
  }
\details{
  This uses the The Independent JPEG Group's JPEG  (libjpeg)  software
  (copyright Thomas G. Lane).
}
  
\value{
  A matrix with rows corresponding
  to the image files given by \code{filenames}
  and columns giving the width and height of the images.
  If there was a problem reading the JPEG file,
  then the width and height are returned as -1 for that
  file and a warning is emitted.
  To determine which files were not read, one can use
  \code{gsub("error in libjpeg for file ", "", names(last.warning))}
}
\references{JPEG}
\author{Duncan Temple Lang <duncan@wald.ucdavis.edu>}
\examples{
\dontrun{
 d = getJPEGDimensions(c("a.jpg", "b.JPEG"))
}

 jpgs = list.files("~", pattern = ".*\.(JPG|jpg)", full.names = TRUE)
 if(length(jpgs))
   getJPEGDimensions(jpgs)
}
\keyword{file}
