\name{getTagInfo}
\alias{getTagInfo}
\title{Get title and description for EXIF tag}
\description{
 This function provides a mechanism to get 
 information about what EXIF tags mean.
 The function consults the libexif library
 to extract fixed strings describing the tags.
 This information is also available if
 one uses the default \code{constructor}
 argument for \code{\link{getExif}}. 
}
\usage{
getTagInfo(tags)
}
\note{
  Using reflectance on the C code, we can provide a
  version of this that avoids the needs to know the names
  but rather can return the information for all known
  tags in the libexif code.
}
\arguments{
  \item{tags}{either a character vector giving the names of the tags
    of interest or an integer vector taking identifying the
    internal enumerated constants (enums) for the the
    desired tags.}
}
\details{
  This is only available in the package if libexif is available
  on your system at installation time of the R package.
}
\value{
  A matrix with 2 columns and as many rows as there are
  elements in the \code{tags} argument.
  The column names are \dQuote{title} and \dQuote{description}.
  The row names are the tags (as strings).
}
\references{
  \url{http://libexif.sourceforge.net}
}
\author{Duncan Temple Lang <duncan@wald.ucdavis.edu>}

\seealso{
 \code{\link{getExif}}
}
\examples{
  pics = getExif(list.files(system.file("images", package = "Rexif"),
                pattern = "\.jpg$", full.names = TRUE))
  tagNames = unique(unlist(lapply(pics, names)))
  getTagInfo(tagNames)
}
\keyword{IO}
\concept{Digital Camera}
