#ifdef HAVE_LIBEXIF

#include <stdio.h>

#include "Rdefines.h"

#include "jpeglib.h"

#include <libexif/exif-data.h>
#include <libexif/exif-entry.h>
#include <libexif/exif-tag.h>

#include "RJPEG.h"


SEXP R_getExif(const char *filename, int fail, SEXP fun);
SEXP R_callRExifEntryConstructor(ExifEntry *entry, const char *filename, SEXP fun);
SEXP R_callRExifMnoteConstructor(ExifMnoteData *nodeData, int  i, const char *filename);
SEXP R_createRExifComponent(ExifData *ex, const char *filename, SEXP fun);
SEXP R_getExifDirect(const char *filename, int fail, SEXP fun);
SEXP R_getExifEntryValue(ExifEntry *entry, const char *filename);

extern void error_handler(j_common_ptr cinfo);


SEXP
R_getExifTags(SEXP file, SEXP failOnError, SEXP direct, SEXP fun)
{
    if(LOGICAL(direct)[0])
       return(R_getExifDirect(CHAR(STRING_ELT(file, 0)), LOGICAL(failOnError)[0], fun));
    else
       return(R_getExif(CHAR(STRING_ELT(file, 0)), LOGICAL(failOnError)[0], fun));
}

SEXP
R_getExifDirect(const char *filename, int fail, SEXP fun)
{
   SEXP ans;
   ExifData *data = exif_data_new_from_file(filename);
   if(!data) {
      PROBLEM  "cannot read Exif from %s", filename
      ERROR;
   }
   ans = R_createRExifComponent(data, filename, fun);
   exif_data_free(data);
   return(ans);
}

SEXP
R_getExif(const char *filename, int fail, SEXP fun)
{
  int i, n;
  FILE * infile;
  SEXP ans, names;
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;
  struct jpeg_marker_struct *ptr;
  R_ErrorInfo errorData;

  cinfo.err = jpeg_std_error(&jerr);
  jerr.error_exit = error_handler; 
  jerr.output_message = error_handler; 

  errorData.fail = fail;
  errorData.filename = filename;
  cinfo.client_data = &errorData;

  jpeg_create_decompress(&cinfo);

  infile = fopen(filename, "rb");
  if(!infile)  {
      return(R_NilValue);
   }
  jpeg_stdio_src(&cinfo, infile);
  jpeg_save_markers(&cinfo, JPEG_COM, 0xFFFF);
  for(i = 0; i < 16; i++)
    jpeg_save_markers(&cinfo, JPEG_APP0 + i, 0xFFFF);
  (void) jpeg_read_header(&cinfo, TRUE);

  ptr = cinfo.marker_list;
  n = 0;

  while(ptr) {
     n++;
     ptr = ptr->next;
  }



  PROTECT(ans = allocVector(VECSXP, n));
  PROTECT(names = allocVector(STRSXP, n));
  ptr = cinfo.marker_list;
  i = 0;
  while(ptr) {
    ExifData *ex;
    ex = exif_data_new_from_data(ptr->data, ptr->data_length);
    if(ex) {
       char buf[10];
       SET_VECTOR_ELT(ans, i, R_createRExifComponent(ex, filename, fun));
       if(ptr->marker == JPEG_COM)
          sprintf(buf, "%s", "JPEG_COM");
       else {
          sprintf(buf, "JPEG_APP%d", (ptr->marker - JPEG_APP0));         
       }
       SET_STRING_ELT(names, i, COPY_TO_USER_STRING(buf));
       i++;
       exif_data_free(ex);
    }

    ptr = ptr->next;
  }
  SET_NAMES(ans, names);

  fclose(infile);
  jpeg_destroy_compress(&cinfo);
  UNPROTECT(2);

  return(ans);
}


SEXP
R_createRExifComponent(ExifData *ex, const char *filename, SEXP fun)
{
       int i, j, n;
       ExifContent *content;

       SEXP ans, names, els, elNames;

       n = EXIF_IFD_COUNT;

       PROTECT(ans = allocVector(VECSXP, n));
       PROTECT(names = allocVector(STRSXP, n));

       for(i = 0 ; i <  n ; i++) {
	 content =  ex->ifd[i];
	 PROTECT(els = allocVector(VECSXP, content->count));
	 PROTECT(elNames = allocVector(STRSXP, content->count));
         for(j = 0 ; j < content->count ; j++) {
   	     ExifEntry *entry;
	     entry = content->entries[j];
	     if(entry->size > 0) {
  	         SET_VECTOR_ELT(els, j, R_callRExifEntryConstructor(entry, filename, fun));
  	         SET_STRING_ELT(elNames, j, mkChar(exif_tag_get_name(entry->tag)));
	     }
	 }
	 SET_NAMES(els, elNames);

	 SET_VECTOR_ELT(ans, i, els);
	 SET_STRING_ELT(names, i, COPY_TO_USER_STRING(exif_ifd_get_name(i)));
	 UNPROTECT(2);
       }
       SET_NAMES(ans, names);
       UNPROTECT(2);

       return(ans);
}


SEXP
R_callRExifEntryConstructor(ExifEntry *entry, const char *filename, SEXP fun)
{
	SEXP e, val, ptr;

        PROTECT(ptr = e = allocVector(LANGSXP, 8)); 
	
	SETCAR(ptr, fun); ptr = CDR(ptr);
	SETCAR(ptr, mkString(exif_tag_get_name(entry->tag))); ptr = CDR(ptr);
	SETCAR(ptr, R_getExifEntryValue(entry, filename)); ptr = CDR(ptr);	


	SETCAR(ptr, ScalarReal(entry->size)); ptr = CDR(ptr);	
	SETCAR(ptr, ScalarInteger(entry->tag)); ptr = CDR(ptr);	
        PROTECT(val = ScalarInteger(entry->format));
	SET_NAMES(val, mkString(exif_format_get_name(entry->format)));
	SETCAR(ptr, val); ptr = CDR(ptr);	
	UNPROTECT(1);

	SETCAR(ptr, mkString(exif_tag_get_title(entry->tag))); ptr = CDR(ptr);
	SETCAR(ptr, mkString(exif_tag_get_description(entry->tag))); ptr = CDR(ptr);

	val = Rf_eval(e, R_GlobalEnv);
	UNPROTECT(1);

	return(val);
}


SEXP
R_getExifEntryValue(ExifEntry *e, const char *filename)
{
   SEXP ans;
   char buf[2000];
   ExifByteOrder o;
   int i;
   

   if(!e || !e->parent || !e->parent->parent)
      return(mkString(exif_entry_get_value(e, buf, sizeof(buf)/sizeof(buf[0]))));

   o = exif_data_get_byte_order (e->parent->parent);
   switch(e->format) {
      case EXIF_FORMAT_LONG:
      {
         long v_long = exif_get_long (e->data, o);              
	 PROTECT(ans = allocVector(REALSXP, e->components));
	 REAL(ans)[0] = v_long;
	 for(i = 1; i < e->components; i++)
  	     REAL(ans)[i] = exif_get_long (e->data + exif_format_get_size (e->format) * i, o);
         UNPROTECT(1);
      }
	 break;
      default:
         exif_entry_get_value(e, buf, sizeof(buf)/sizeof(buf[0]));
         ans = mkString(buf[0] ? buf : "");
	 break;
   }

   return(ans);
}


SEXP
R_lookupTagByName(SEXP names)
{
   SEXP ans;
   int i, n;
 
   n = GET_LENGTH(names);
   ans = allocVector(INTSXP, n);
   for(i = 0; i < n ; i++)
	INTEGER(ans)[i] = exif_tag_from_name(CHAR(STRING_ELT(names, i)));

   return(ans);
}

SEXP 
R_getTagInfo(SEXP tags)
{
   SEXP ans;
   int i, n;
 
   n = GET_LENGTH(tags);
   ans = allocVector(STRSXP, 2 * n);
   for(i = 0; i < n ; i += 2) {
        const char *tmp;
	tmp = exif_tag_get_title(INTEGER(tags)[i]);
	SET_STRING_ELT(ans, i, tmp ? mkChar((char *) tmp) : R_NaString);
	tmp = exif_tag_get_description(INTEGER(tags)[i]);
	SET_STRING_ELT(ans, i + 1, tmp ? mkChar((char *) tmp) : R_NaString);
   }

   return(ans);  
}


#if 0
SEXP
R_createRExifComponent(ExifData *ex, const char *filename)
{
       int i, j, n;
       ExifContent *content;
       ExifMnoteData *mnote;

       SEXP ans, names;
       const char *tmp;

       mnote = exif_data_get_mnote_data(ex);
       n = exif_mnote_data_count(mnote);

       PROTECT(ans = allocVector(VECSXP, n));
       PROTECT(names = allocVector(STRSXP, n));

       for(i = 0 ; i <  n ; i++) {
          SET_VECTOR_ELT(ans, i, R_callRExifMnoteConstructor(mnote, i, filename));
	  tmp = exif_mnote_data_get_name(mnote, i);
	  SET_STRING_ELT(names, i, COPY_TO_USER_STRING(tmp ? tmp : ""));
       }
       SET_NAMES(ans, names);
       UNPROTECT(2);

       return(ans);
}



SEXP
R_callRExifMnoteConstructor(ExifMnoteData *noteData, int  i, const char *filename)
{
	SEXP e, ans, val, ptr;
	char buf[2000];
        const char *tmp;

        PROTECT(ptr = e = allocVector(LANGSXP, 6)); 
	
	SETCAR(ptr, Rf_install("createExifMnote")); ptr = CDR(ptr);
	tmp = exif_mnote_data_get_name(noteData, i);
	SETCAR(ptr, mkString(tmp ? tmp : "")); ptr = CDR(ptr);
	SETCAR(ptr, mkString(exif_mnote_data_get_value(noteData, i, buf, sizeof(buf)/sizeof(buf[0])))); ptr = CDR(ptr);	
	SETCAR(ptr, ScalarReal(exif_mnote_data_get_id(noteData, i))); ptr = CDR(ptr);	

	tmp = exif_mnote_data_get_title(noteData, i);
	SETCAR(ptr, mkString(tmp ? tmp : "")); ptr = CDR(ptr);
	tmp = exif_mnote_data_get_description(noteData, i);
	SETCAR(ptr, mkString(tmp ? tmp : "")); ptr = CDR(ptr);

	val = Rf_eval(e, R_GlobalEnv);
	UNPROTECT(1);

	return(val);
}
#endif


#else 

void
__foo__()
{

}
#endif

